@@ -1,10 +1,11 @@
 package Mongoose;
 BEGIN {
-  $Mongoose::VERSION = '0.02';
+  $Mongoose::VERSION = '0.03';
 }
 use MongoDB;
 use MooseX::Singleton;
 use Mongoose::Join;
+use Mongoose::File;
 use Mongoose::Meta::AttributeTraits;
 use Carp;
 
@@ -76,7 +77,7 @@ Mongoose - MongoDB document to Moose object mapper
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -229,6 +230,10 @@ L<KiokuDB>
 
 	Rodrigo de Oliveira (rodrigolive), C<rodrigolive@gmail.com>
 
+=head1 CONTRIBUTORS
+
+	Arthur Wolf
+
 =head1 LICENSE
 
 This library is free software. You can redistribute it and/or modify it under
